defmodule GameCoordinatorTestGame do
  @behaviour Amethyst.Game

  @moduledoc """
  This module is a sample game for the purpose of
  testing the GameCoordinator
  """

  @impl true
  def instantiate(supervisor) do
    {:ok, %{test: "test"}}
  end

  @impl true
  def login(from, cfg, refs) do
    :accept
  end

  @impl true
  def joinable?(refs) do
    true
  end

  @impl true
  def player_position(from, {x, y, z}, refs) do
    :ok
  end
end

defmodule GameCoordinatorTest do
  use ExUnit.Case, async: true

  @moduledoc """
  This module includes tests for Amethyst.GameCoordinator
  """

  test "Create an instance of a game with create/1 then try to find it" do
    %Amethyst.GameCoordinator.Game{refs: refs, gid: gid, mod: mod} = Amethyst.GameCoordinator.create(GameCoordinatorTestGame)
    assert mod == GameCoordinatorTestGame
    assert refs[:test] == "test"

    found_game = Amethyst.GameCoordinator.find(GameCoordinatorTestGame)
    assert found_game.mod == GameCoordinatorTestGame
    assert found_game.refs[:test] == "test"
    assert found_game.gid == gid
    Amethyst.GameCoordinator.remove(gid)
  end

  test "Create an instance of a game with find_or_create/1 and then find it with find_or_create/1" do
    %Amethyst.GameCoordinator.Game{refs: refs, gid: gid, mod: mod} = Amethyst.GameCoordinator.find_or_create(GameCoordinatorTestGame)
    assert mod == GameCoordinatorTestGame
    assert refs[:test] == "test"

    found_game = Amethyst.GameCoordinator.find_or_create(GameCoordinatorTestGame)
    assert found_game.mod == GameCoordinatorTestGame
    assert found_game.refs[:test] == "test"
    assert found_game.gid == gid
    Amethyst.GameCoordinator.remove(gid)
  end
end
